@@ -2,12 +2,13 @@
 /**
  *  @file autogen.c
  *
- *  Time-stamp:        "2012-01-07 10:40:56 bkorb"
- *
  *  This is the main routine for autogen.
- *
- *  This file is part of AutoGen.
- *  Copyright (c) 1992-2012 Bruce Korb - all rights reserved
+ *  @addtogroup autogen
+ *  @{
+ */
+
+/*  This file is part of AutoGen.
+ *  Copyright (C) 1992-2013 Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -37,8 +38,8 @@ typedef enum {
 static char const * const state_names[] = { STATE_TABLE };
 #undef _State_
 
-static sigjmp_buf  abendJumpEnv;
-static int         abendJumpSignal = 0;
+static sigjmp_buf  abend_env;
+static int         abend_sig = 0;
 
 typedef void (sighandler_proc_t)(int sig);
 static sighandler_proc_t ignore_signal, catch_sig_and_bail;
@@ -68,8 +69,13 @@ setup_signals(sighandler_proc_t * chldHandler,
               sighandler_proc_t * dfltHandler);
 /* = = = END-STATIC-FORWARD = = = */
 
+#ifndef HAVE_CHMOD
+#  include "compat/chmod.c"
+#endif
+
 /**
  * main routine under Guile guidance
+ *
  * @param  closure Guile closure parameter.  Not used.
  * @param  argc    argument count
  * @param  argv    argument vector
@@ -80,34 +86,36 @@ inner_main(void * closure, int argc, char ** argv)
     atexit(done_check);
     initialize(argc, argv);
 
-    procState = PROC_STATE_LOAD_DEFS;
-    readDefines();
+    processing_state = PROC_STATE_LOAD_DEFS;
+    read_defs();
 
     /*
      *  Activate the AutoGen specific Scheme functions.
      *  Then load, process and unload the main template
      */
-    procState = PROC_STATE_LOAD_TPL;
+    processing_state = PROC_STATE_LOAD_TPL;
 
     {
-        tTemplate* pTF = loadTemplate(pzTemplFileName, NULL);
+        templ_t* pTF = tpl_load(tpl_fname, NULL);
 
-        procState = PROC_STATE_EMITTING;
-        processTemplate(pTF);
+        processing_state = PROC_STATE_EMITTING;
+        process_tpl(pTF);
 
-        procState = PROC_STATE_CLEANUP;
+        processing_state = PROC_STATE_CLEANUP;
         cleanup(pTF);
     }
 
-    procState = PROC_STATE_DONE;
+    processing_state = PROC_STATE_DONE;
     setup_signals(SIG_DFL, SIG_IGN, SIG_DFL);
     exit_code = AUTOGEN_EXIT_SUCCESS;
     done_check();
     /* NOTREACHED */
+    (void)closure;
 }
 
 /**
  * main() called from _start()
+ *
  * @param  argc    argument count
  * @param  argv    argument vector
  * @return nothing -- Guile never returns, but calls exit(2).
@@ -119,27 +127,13 @@ main(int argc, char ** argv)
      *  IF we've been kicked with a signal,
      *  THEN abort, passing the signal that whacked us.
      */
-    if (sigsetjmp(abendJumpEnv, 0) != 0)
-        cleanup_and_abort(abendJumpSignal);
+    if (sigsetjmp(abend_env, 0) != 0)
+        cleanup_and_abort(abend_sig);
 
     setup_signals(ignore_signal, SIG_DFL, catch_sig_and_bail);
     optionSaveState(&autogenOptions);
-    pfTrace = stderr;
-
-    /*
-     *  as of 2.0.2, Guile will fiddle with strings all on its own accord.
-     *  Coerce the environment into being POSIX ASCII strings so it keeps
-     *  its bloody stinking nose out of our data.
-     */
-    putenv((char*)(void*)LC_ALL_IS_C);
-
-    /*
-     *  If GUILE_WARN_DEPRECATED has not been defined, then likely we are
-     *  not in a development environment and likely we don't want to give
-     *  our users any angst.
-     */
-    if (getenv(GUILE_WARN_DEP_STR) == NULL)
-        putenv((char*)(void*)GUILE_WARN_NO_ENV);
+    trace_fp = stderr;
+    prep_env();
 
     AG_SCM_BOOT_GUILE(argc, argv, inner_main);
 
@@ -148,7 +142,9 @@ main(int argc, char ** argv)
 }
 
 /**
- *  This code must run regardless of which exit path is taken
+ * This code must run regardless of which exit path is taken
+ *
+ * @param [in] cl_wait Whether or not a child process should be waited for.
  */
 static void
 exit_cleanup(wait_for_pclose_enum_t cl_wait)
@@ -160,7 +156,7 @@ exit_cleanup(wait_for_pclose_enum_t cl_wait)
         static int exit_cleanup_done = 0;
         if (exit_cleanup_done) {
             if (OPT_VALUE_TRACE > TRACE_NOTHING)
-                fprintf(pfTrace, EXIT_CLEANUP_MULLIGAN);
+                fprintf(trace_fp, EXIT_CLEANUP_MULLIGAN);
             return;
         }
 
@@ -174,28 +170,28 @@ exit_cleanup(wait_for_pclose_enum_t cl_wait)
     close_server_shell();
 
     if (OPT_VALUE_TRACE > TRACE_NOTHING)
-        fprintf(pfTrace, EXIT_CLEANUP_DONE_FMT,
+        fprintf(trace_fp, EXIT_CLEANUP_DONE_FMT,
                 (cl_wait == EXIT_PCLOSE_WAIT)
                 ? EXIT_CLEANUP_WAITED : EXIT_CLEANUP_NOWAIT,
                 (unsigned int)getpid());
 
     do  {
-        if (pfTrace == stderr)
+        if (trace_fp == stderr)
             break;
 
         if (! trace_is_to_pipe) {
-            fclose(pfTrace);
+            fclose(trace_fp);
             break;
         }
 
 
-        fflush(pfTrace);
-        pclose(pfTrace);
+        fflush(trace_fp);
+        pclose(trace_fp);
         if (cl_wait == EXIT_PCLOSE_WAIT) {
             int status;
             while (wait(&status) > 0)  ;
         }
-    } while (0);
+    } while (false);
 
     fflush(stdout);
     fflush(stderr);
@@ -204,51 +200,59 @@ exit_cleanup(wait_for_pclose_enum_t cl_wait)
 /**
  *  A signal was caught, siglongjmp called and main() has called this.
  *  We do not deallocate stuff so it can be found in the core dump.
+ *
+ *  @param[in] sig the signal number
  */
 static void
 cleanup_and_abort(int sig)
 {
-    if (*pzOopsPrefix != NUL) {
-        fputs(pzOopsPrefix, stderr);
-        pzOopsPrefix = zNil;
+    int sig_exit_code = AUTOGEN_EXIT_SIGNAL + sig;
+    if (processing_state == PROC_STATE_INIT) {
+        fprintf(stderr, AG_NEVER_STARTED, sig, strsignal(sig));
+        _exit(sig_exit_code);
+    }
+
+    if (*oops_pfx != NUL) {
+        fputs(oops_pfx, stderr);
+        oops_pfx = zNil;
     }
 
     fprintf(stderr, AG_SIG_ABORT_FMT, sig, strsignal(sig),
-            ((unsigned)procState <= PROC_STATE_DONE)
-            ? state_names[ procState ] : BOGUS_TAG);
+            ((unsigned)processing_state <= PROC_STATE_DONE)
+            ? state_names[ processing_state ] : BOGUS_TAG);
 
-    if (procState == PROC_STATE_ABORTING) {
+    if (processing_state == PROC_STATE_ABORTING) {
         exit_cleanup(EXIT_PCLOSE_NOWAIT);
-        _exit(sig + 128);
+        _exit(sig_exit_code);
     }
 
-    procState = PROC_STATE_ABORTING;
+    processing_state = PROC_STATE_ABORTING;
     setup_signals(SIG_DFL, SIG_DFL, SIG_DFL);
 
     /*
      *  IF there is a current template, then we should report where we are
      *  so that the template writer knows where to look for their problem.
      */
-    if (pCurTemplate != NULL) {
+    if (current_tpl != NULL) {
         int line;
-        teFuncType fnCd;
+        mac_func_t fnCd;
         char const * pzFn;
         char const * pzFl;
 
-        if (pCurMacro == NULL) {
+        if (cur_macro == NULL) {
             pzFn = PSEUDO_MACRO_NAME_STR;
             line = 0;
-            fnCd = -1;
+            fnCd = (mac_func_t)-1;
 
         } else {
-            teFuncType f =
-                (pCurMacro->funcCode > FUNC_CT)
-                    ? FTYP_SELECT : pCurMacro->funcCode;
-            pzFn = apzFuncNames[ f ];
-            line = pCurMacro->lineNo;
-            fnCd = pCurMacro->funcCode;
+            mac_func_t f =
+                (cur_macro->md_code > FUNC_CT)
+                    ? FTYP_SELECT : cur_macro->md_code;
+            pzFn = ag_fun_names[ f ];
+            line = cur_macro->md_line;
+            fnCd = cur_macro->md_code;
         }
-        pzFl = pCurTemplate->pzTplFile;
+        pzFl = current_tpl->td_file;
         if (pzFl == NULL)
             pzFl = NULL_FILE_NAME_STR;
         fprintf(stderr, AG_ABORT_LOC_FMT, pzFl, line, pzFn, fnCd);
@@ -276,21 +280,23 @@ cleanup_and_abort(int sig)
  *  catch_sig_and_bail catches signals we abend on.  The "siglongjmp"
  *  goes back to the real "main()" procedure and it will call
  *  "cleanup_and_abort()", above.
+ *
+ *  @param[in] sig the signal number
  */
 static void
 catch_sig_and_bail(int sig)
 {
-    switch (procState) {
+    switch (processing_state) {
     case PROC_STATE_ABORTING:
-        exit_code = AUTOGEN_EXIT_SIGNAL;
+        exit_code = AUTOGEN_EXIT_SIGNAL + sig;
 
     case PROC_STATE_DONE:
         break;
 
     default:
-        abendJumpSignal = sig;
-        exit_code = AUTOGEN_EXIT_SIGNAL;
-        siglongjmp(abendJumpEnv, sig);
+        abend_sig = sig;
+        exit_code = AUTOGEN_EXIT_SIGNAL + sig;
+        siglongjmp(abend_env, sig);
     }
 }
 
@@ -300,10 +306,13 @@ catch_sig_and_bail(int sig)
  *  it will kill us.  If we set it to ignore, it will be inherited.
  *  Therefore, always in all programs set it to call a procedure.
  *  The "wait(3)" call will do magical things, but will not override SIGIGN.
+ *
+ *  @param[in] sig the signal number
  */
 static void
 ignore_signal(int sig)
 {
+    (void)sig;
     return;
 }
 
@@ -325,44 +334,44 @@ done_check(void)
         static int done_check_done = 0;
         if (done_check_done) {
             if (OPT_VALUE_TRACE > TRACE_NOTHING)
-                fprintf(pfTrace, DONE_CHECK_REDONE);
+                fprintf(trace_fp, DONE_CHECK_REDONE);
             return;
         }
         done_check_done = 1;
     }
 
-    switch (procState) {
+    switch (processing_state) {
     case PROC_STATE_EMITTING:
     case PROC_STATE_INCLUDING:
         /*
-         *  A library (viz., Guile) procedure has called exit(3C).
-         *  The AutoGen abort paths all set procState to PROC_STATE_ABORTING.
+         * A library (viz., Guile) procedure has called exit(3C).  The
+         * AutoGen abort paths all set processing_state to PROC_STATE_ABORTING.
          */
-        if (*pzOopsPrefix != NUL) {
+        if (*oops_pfx != NUL) {
             /*
              *  Emit the CGI page header for an error message.  We will rewind
              *  stderr and write the contents to stdout momentarily.
              */
-            fputs(pzOopsPrefix, stdout);
-            pzOopsPrefix = zNil;
+            fputs(oops_pfx, stdout);
+            oops_pfx = zNil;
         }
 
         if (OPT_VALUE_TRACE > TRACE_NOTHING)
             scm_backtrace();
 
-        fprintf(stderr, SCHEME_EVAL_ERR_FMT, pCurTemplate->pzTplFile,
-                pCurMacro->lineNo);
+        fprintf(stderr, SCHEME_EVAL_ERR_FMT, current_tpl->td_file,
+                cur_macro->md_line);
 
         /*
          *  We got here because someone called exit early.
          */
         do  {
 #ifndef DEBUG_ENABLED
-            out_close(AG_FALSE);
+            out_close(false);
 #else
-            out_close(AG_TRUE);
+            out_close(true);
 #endif
-        } while (pCurFp->pPrev != NULL);
+        } while (cur_fpstack->stk_prev != NULL);
         exit_code = AUTOGEN_EXIT_BAD_TEMPLATE;
         break; /* continue failure exit */
 
@@ -371,20 +380,20 @@ done_check(void)
         /* FALLTHROUGH */
 
     default:
-        fprintf(stderr, AG_ABEND_STATE_FMT, state_names[procState]);
+        fprintf(stderr, AG_ABEND_STATE_FMT, state_names[processing_state]);
         goto autogen_aborts;
 
     case PROC_STATE_ABORTING:
         exit_code = AUTOGEN_EXIT_BAD_TEMPLATE;
 
     autogen_aborts:
-        if (*pzOopsPrefix != NUL) {
+        if (*oops_pfx != NUL) {
             /*
              *  Emit the CGI page header for an error message.  We will rewind
              *  stderr and write the contents to stdout momentarily.
              */
-            fputs(pzOopsPrefix, stdout);
-            pzOopsPrefix = zNil;
+            fputs(oops_pfx, stdout);
+            oops_pfx = zNil;
         }
         break; /* continue failure exit */
 
@@ -396,8 +405,8 @@ done_check(void)
         break; /* continue normal exit */
     }
 
-    if (pzLastScheme != NULL)
-        fprintf(stderr, GUILE_CMD_FAIL_FMT, pzLastScheme);
+    if (last_scm_cmd != NULL)
+        fprintf(stderr, GUILE_CMD_FAIL_FMT, last_scm_cmd);
 
     /*
      *  IF we diverted stderr, then now is the time to copy the text to stdout.
@@ -419,7 +428,7 @@ done_check(void)
             fread( pz, (size_t)1, (size_t)pos, stderr);
             fwrite(pz, (size_t)1, (size_t)pos, stdout);
             AGFREE(pz);
-        } while (0);
+        } while (false);
 
         unlink(cgi_stderr);
         AGFREE(cgi_stderr);
@@ -429,7 +438,7 @@ done_check(void)
     ag_scribble_deinit();
 
     if (OPT_VALUE_TRACE > TRACE_NOTHING)
-        fprintf(pfTrace, DONE_CHECK_DONE);
+        fprintf(trace_fp, DONE_CHECK_DONE);
 
     exit_cleanup(EXIT_PCLOSE_WAIT);
 
@@ -439,7 +448,7 @@ done_check(void)
      *  exit code.  (Always EXIT_FAILURE, unless this was called at the end
      *  of inner_main().)
      */
-    if (procState != PROC_STATE_OPTIONS)
+    if (processing_state != PROC_STATE_OPTIONS)
         _exit(exit_code);
 }
 
@@ -451,18 +460,18 @@ ag_abend_at(char const * pzMsg
 #endif
     )
 {
-    if (*pzOopsPrefix != NUL) {
-        fputs(pzOopsPrefix, stderr);
-        pzOopsPrefix = zNil;
+    if (*oops_pfx != NUL) {
+        fputs(oops_pfx, stderr);
+        oops_pfx = zNil;
     }
 
 #ifdef DEBUG_ENABLED
     fprintf(stderr, "Giving up in %s line %d\n", pzFile, line);
 #endif
 
-    if ((procState >= PROC_STATE_LIB_LOAD) && (pCurTemplate != NULL)) {
-        int l_no = (pCurMacro == NULL) ? -1 : pCurMacro->lineNo;
-        fprintf(stderr, ERROR_IN_TPL_FMT, pCurTemplate->pzTplFile, l_no);
+    if ((processing_state >= PROC_STATE_LIB_LOAD) && (current_tpl != NULL)) {
+        int l_no = (cur_macro == NULL) ? -1 : cur_macro->md_line;
+        fprintf(stderr, ERROR_IN_TPL_FMT, current_tpl->td_file, l_no);
     }
     fputs(pzMsg, stderr);
     pzMsg += strlen(pzMsg);
@@ -470,14 +479,14 @@ ag_abend_at(char const * pzMsg
         fputc(NL, stderr);
 
     {
-        teProcState oldState = procState;
-        procState = PROC_STATE_ABORTING;
+        proc_state_t oldState = processing_state;
+        processing_state = PROC_STATE_ABORTING;
 
         switch (oldState) {
         case PROC_STATE_EMITTING:
         case PROC_STATE_INCLUDING:
         case PROC_STATE_CLEANUP:
-            longjmp(fileAbort, FAILURE);
+            longjmp(abort_jmp_buf, FAILURE);
             /* NOTREACHED */
         default:
             exit(EXIT_FAILURE);
@@ -536,6 +545,9 @@ setup_signals(sighandler_proc_t * chldHandler,
              */
 #ifdef SIGTSTP
         case SIGTSTP:
+#endif
+#ifdef SIGPROF
+        case SIGPROF:
 #endif
             continue;
 
@@ -632,7 +644,9 @@ ao_strdup (char const * str)
         (void)option_load_mode, (void)program_pkgdatadir;
     }
 #endif
-/*
+/**
+ * @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
